local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local _

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Glyphs tested:
-- Everlasting Affliction
-- Dark Soul
-- Life Tap
-- Burning Embers
-- Conflagrate
-- Verdant Spheres

local SpellPowerMultiplier = {
	1459--[[Arcane Brilliance]],
	61316--[[Dalaran Brilliance]],
	77747--[[Burning Wrath]],
	109773--[[Dark Intent]],
	126309--[[Still Water]],
}

local MagicVulnerability = {
	1490--[[Curse of the Elements]],
	58410--[[Master Poisoner]],
	116202--[[Aura of the Elements]],
	34889--[[Fire Breath]],
	24844--[[Lightning Breath]],
}

local StaminaBuff = {
	469--[[Commanding Shout]],
	21562--[[Power Word: Fortitude]],
	90364--[[Qiraji Fortitude]],
	109773--[[Dark Intent]],
}

a.spells["Pet Attack"] = {
	Override = function(z)
		if x.NoCC and x.PetAlive and x.Enemy and not UnitGUID("pettarget") then
			s.FlashPet(PET_ACTION_ATTACK, "Green")
		elseif x.NoCC and x.PetAlive and x.Enemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK, "Pink")
		end
	end,
}

a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy and not a.PetAssistActive() then
			s.FlashPet(PET_MODE_ASSIST, "Pink")
		end
	end,
}

a.spells["Dark Intent"] = {
	ID = 109773--[[Dark Intent]],
	NotWhileMoving = false,
	EarlyRefresh = 600,
	NoRangeCheck = true,
	FlashColor = "Green",
	Check = function(z)
		return not s.Buff(SpellPowerMultiplier, "raid") or not s.Buff(StaminaBuff, "raid")
	end,
}

a.spells["Curse of the Elements"] = {
	ID = 1490--[[Curse of the Elements]],
	Debuff = MagicVulnerability,
	EarlyRefresh = 5,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Check = function(z)
		if s.Boss("target") then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.NoCC and not x.Metamorphosis
	end,
}

a.spells["Aura of the Elements"] = {
	ID = 1490--[[Curse of the Elements]],
	Buff = 116202--[[Aura of the Elements]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.Metamorphosis
	end,
	FlashColor = "Blue",
	Continue = true,
}

a.spells["Summon Felhunter ooc"] = {
	ID = 691--[[Summon Felhunter]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ShouldPermanentBuff and not x.PetAlive and not s.Buff({132614--[[Grimoire of Sacrifice]], 132612--[[Grimoire of Sacrifice]]}, "player")
	end,
	FlashID = {691--[[Summon Felhunter]], 112869--[[Summon Observer]], 10--[[Summon Demon]]}
}

a.spells["Summon Felguard ooc"] = {
	ID = 30146--[[Summon Felguard]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ShouldPermanentBuff and not x.PetAlive and not s.Buff(132613--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {30146--[[Summon Felguard]], 112870--[[Summon Wrathguard]], 10--[[Summon Demon]]}
}

a.spells["Dark Soul: Misery"] = {
	ID = 113860--[[Dark Soul: Misery]],
	MyBuff = 113860--[[Dark Soul: Misery]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		if s.Boss("target") then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.ShouldTemporaryBuff and x.InCombat and x.ActiveEnemy and s.Debuff(MagicVulnerability, "target")
	end,
	NoGCD = true,
}

a.spells["Dark Soul: Knowledge"] = {
	ID = 113861--[[Dark Soul: Knowledge]],
	MyBuff = 113861--[[Dark Soul: Knowledge]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		if s.Boss("target") then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.ShouldTemporaryBuff and x.InCombat and x.ActiveEnemy and s.Debuff(MagicVulnerability, "target")
	end,
	NoGCD = true,
}

a.spells["Dark Soul: Instability"] = {
	ID = 113858--[[Dark Soul: Instability]],
	MyBuff = 113858--[[Dark Soul: Instability]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		if s.Boss("target") then
			z.FlashColor = "Yellow"
			z.Continue = false
		else
			z.FlashColor = "Pink"
			z.Continue = true
		end
		return x.ShouldTemporaryBuff and x.InCombat and x.ActiveEnemy and s.Debuff(MagicVulnerability, "target")
	end,
	NoGCD = true,
}

a.spells["Grimoire: Felhunter"] = {
	ID = 111897--[[Grimoire: Felhunter]],
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashID = {111897--[[Grimoire: Felhunter]], 108501--[[Grimoire of Service]]},
}

a.spells["Grimoire: Felguard"] = {
	ID = 111898--[[Grimoire: Felguard]],
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and s.HasTalent(108501--[[Grimoire of Service]])
	end,
	FlashID = {111898--[[Grimoire: Felguard]], 108501--[[Grimoire of Service]]},
}

a.spells["Grimoire of Sacrifice"] = {
	ID = 108503--[[Grimoire of Sacrifice]],
	MyBuff = {132612--[[Grimoire of Sacrifice]], 132613--[[Grimoire of Sacrifice]], 132614--[[Grimoire of Sacrifice]]},
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff and x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]])
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Summon Felhunter (Sacrifice down)"] = {
	ID = 691--[[Summon Felhunter]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and not x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]]) and not s.Buff(108503--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {691--[[Summon Felhunter]], 10--[[Summon Demon]]}
}

a.spells["Summon Felguard (Sacrifice down)"] = {
	ID = 30146--[[Summon Felguard]],
	NotWhileMoving = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and not x.PetAlive and s.HasTalent(108503--[[Grimoire of Sacrifice]]) and not s.Buff(108503--[[Grimoire of Sacrifice]], "player")
	end,
	FlashID = {30146--[[Summon Felguard]], 10--[[Summon Demon]]}
}

a.spells["Summon Doomguard"] = {
	ID = 18540--[[Summon Doomguard]],
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and s.Boss("target")
	end,
	FlashColor = "Purple",
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Summon Terrorguard"] = {
	ID = 112927--[[Summon Terrorguard]],
	Check = function(z)
		return x.ActiveEnemy and x.InCombat and s.Boss("target")
	end,
	FlashColor = "Purple",
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Soul Swap with Soulburn"] = {
	ID = 86121--[[Soul Swap]],
	ConsumeBuff = 74434--[[Soulburn]],
	ConsumeBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Soulburn w/ Dark Soul or Trinket proc"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
	-- (buff.dark_soul.up|trinket.proc.intellect.react|trinket.stacking_proc.intellect.react>7)&(dot.agony.ticks_remain<=action.agony.add_ticks%2|dot.corruption.ticks_remain<=action.corruption.add_ticks%2|dot.unstable_affliction.ticks_remain<=action.unstable_affliction.add_ticks%2)&shard_react
		return x.ActiveEnemy and x.InCombat and GetSpellBonusDamage(6) > max(a.getDoTStat(980--[[Agony]], "spell_power"), a.getDoTStat(30108--[[Unstable Affliction]], "spell_power")) and (x.DarkSoulRemains > x.GCD or x.IntProc > x.GCD or x.IntProcStack > 7) and (x.AgonyTicksRemain * 2 <= x.AgonyTicks or x.CorruptionTicksRemain * 2 <= x.CorruptionTicks or x.UATicksRemain * 2 <= x.UATicks)
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Soulburn under 20%"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
	-- (dot.unstable_affliction.ticks_remain<=1|dot.corruption.ticks_remain<=1|dot.agony.ticks_remain<=1)&shard_react&target.health.pct<=20
		return x.ActiveEnemy and x.InCombat and s.HealthPercent("target") <= 20 and (x.UATicksRemain <= 1 or x.CorruptionTicksRemain <= 1 or x.AgonyTicksRemain <= 1)
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Soulburn under 20% w/ procs"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
	-- stat.spell_power>dot.unstable_affliction.spell_power&dot.unstable_affliction.ticks_remain<=action.unstable_affliction.add_ticks%2&shard_react&target.health.pct<=20
		return x.ActiveEnemy and x.InCombat and GetSpellBonusDamage(6) > a.getDoTStat(30108--[[Unstable Affliction]], "spell_power") and x.UATicksRemain * 2 <= x.UATicks and s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Haunt under 20%"] = {
	ID = 48181--[[Haunt]],
	MyDebuff = 48181--[[Haunt]],
	NotWhileMoving = true,
	Check = function(z)
	-- !in_flight_to_target&remains<cast_time+travel_time+tick_time&shard_react&target.health.pct<=20
		z.EarlyRefresh = (s.LastSpellTravelTime(48181--[[Haunt]], "target") or 1) + x.TickSpeed + s.CastTime(48181--[[Haunt]])
		return x.NoCC and not s.SpellOrAuraDelay(48181--[[Haunt]], "target", true) and s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
}

a.spells["Haunt"] = {
	ID = 48181--[[Haunt]],
	MyDebuff = 48181--[[Haunt]],
	NotWhileMoving = true,
	Check = function(z)
	-- !in_flight_to_target&remains<cast_time+travel_time+tick_time&(soul_shard>2|cooldown.dark_soul.remains>35|(soul_shard>1&cooldown.dark_soul.remains<cast_time))&shard_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		z.EarlyRefresh = (s.LastSpellTravelTime(48181--[[Haunt]], "target") or 1) + x.TickSpeed + s.CastTime(48181--[[Haunt]])
		return x.NoCC and not s.SpellOrAuraDelay(48181--[[Haunt]], "target", true) and (x.SoulShards > 2 or x.DarkSoulCD > 35 or (x.SoulShards > 1 and x.DarkSoulCD < s.CastTime(48181--[[Haunt]])))
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony when running out"] = {
	ID = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
	-- remains<gcd&remains+2<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and x.AgonyDuration < (x.MaxGCD + x.GCD) and x.AgonyDuration + 2 < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony"] = {
	ID = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
	-- ticks_remain<=2&remains+2<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and (x.AgonyTicksRemain <= 2 or x.AgonyDuration < s.CastTime(103103--[[Malefic Grasp]]) + x.GCD) and x.AgonyDuration + 2 < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["Agony w/ procs"] = {
	ID = 980--[[Agony]],
	NotWhileMoving = false,
	Check = function(z)
	-- stat.spell_power>spell_power&(!trinket.stacking_proc.intellect.react|trinket.stacking_proc.intellect.react>7)&ticks_remain<add_ticks%2&remains+2<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and GetSpellBonusDamage(6) > a.getDoTStat(980--[[Agony]], "spell_power") and (x.IntProcStack == 0 or x.IntProcStack > 7) and x.AgonyTicksRemain * 2 < x.AgonyTicks and x.AgonyDuration + 2 < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption when running out"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
	-- remains<gcd&remains<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		z.EarlyRefresh = min((x.MaxGCD + x.GCD), x.DarkSoulCD)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption"] = {
	ID = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
	-- ticks_remain<=2&remains<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and x.CorruptionTicksRemain <= 2 and x.CorruptionDuration < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption w/ procs"] = {
	ID = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
	-- stat.spell_power>spell_power&(!trinket.stacking_proc.intellect.react|trinket.stacking_proc.intellect.react>7)&ticks_remain<add_ticks%2&remains<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and GetSpellBonusDamage(6) > a.getDoTStat(172--[[Corruption]], "spell_power") and (x.IntProcStack == 0 or x.IntProcStack > 7) and x.CorruptionTicksRemain * 2 < x.CorruptionTicks and x.CorruptionDuration < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption (Demo) w/ Perfect Aim"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and not x.Metamorphosis and x.PerfectAimDuration > x.GCD and (a.getDoTStat(172--[[Corruption]], "crit_pct") < 100 or x.CorruptionTicksRemain <= x.CorruptionTicks)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Corruption (Demo)"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and x.TimeToDie >= 6 and not x.Metamorphosis
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction when running out"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
	-- remains<gcd+cast_time&remains<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		z.EarlyRefresh = min((x.MaxGCD + x.GCD + s.CastTime(30108--[[Unstable Affliction]])), x.DarkSoulCD)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction"] = {
	ID = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
	-- (remains-cast_time)%(duration%current_ticks)<=2&remains<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and (x.UADuration - s.CastTime(30108--[[Unstable Affliction]])) * x.UATicks / x.UADuration <= 2 and x.UADuration < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["Unstable Affliction w/ procs"] = {
	ID = 30108--[[Unstable Affliction]],
	NotWhileMoving = true,
	Check = function(z)
	-- stat.spell_power>spell_power&(!trinket.stacking_proc.intellect.react|trinket.stacking_proc.intellect.react>7)&ticks_remain<add_ticks%2&remains<cooldown.dark_soul.remains&miss_react
		if x.InterruptMaleficGrasp then
			z.FlashColor = "white"
		else
			z.FlashColor = "Pink"
		end
		return x.NoCC and GetSpellBonusDamage(6) > a.getDoTStat(30108--[[Unstable Affliction]], "spell_power") and (x.IntProcStack == 0 or x.IntProcStack > 7) and x.UATicksRemain * 2 < x.UATicks and x.UADuration < x.DarkSoulCD
	end,
	EnemyTargetNeeded = true,
}

a.spells["Shadowburn"] = {
	ID = 17877--[[Shadowburn]],
	Check = function(z)
		return x.NoCC and (x.BurningEmbers > 3.5 or s.PowerPercent("player", SPELL_POWER_MANA) <= 20 or x.DarkSoulRemains > x.GCD or x.TimeToDie < 20 or s.BuffStack(80240--[[Havoc]], "player") >= 1)
	end,
	FlashColor = "Orange",
	EnemyTargetNeeded = true,
}

a.spells["Chaos Bolt w/ 3 stacks Havoc"] = {
	ID = 116858--[[Chaos Bolt]],
	ConsumeBuff = 80240--[[Havoc]],
	ConsumeStack = 3,
	ConsumeBuffUnit = "player",
	Check = function(z)
		return x.NoCC and s.HealthPercent("target") > 20 and s.BuffDuration(80240--[[Havoc]], "player") > s.CastTime(116858--[[Chaos Bolt]])
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Chaos Bolt"] = {
	ID = 116858--[[Chaos Bolt]],
	Check = function(z)
		local cast_time = s.CastTime(116858--[[Chaos Bolt]])
		return x.NoCC and s.HealthPercent("target") > 20 and (s.BuffStack(117828--[[Backdraft]], "player") < 3 or (x.Enemies > 1 and s.CastTime(29722--[[Incinerate]]) < 1)) and (x.BurningEmbers > (4.5 - x.Enemies) or x.DarkSoulRemains > cast_time or s.BuffDuration(114206--[[Skull Banner]], "player") > cast_time or (x.IntProc > cast_time))
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Conflagrate w/ 2 charges"] = {
	ID = 17962--[[Conflagrate]],
--	MyBuff = 117828--[[Backdraft]],
--	StackGiven = 3,
	BuffUnit = "player",
	Check = function(z)
		return x.NoCC and not s.Buff(80240--[[Havoc]], "player") and GetSpellCharges(17962--[[Conflagrate]]) == 2
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Conflagrate"] = {
	ID = 17962--[[Conflagrate]],
--	MyBuff = 117828--[[Backdraft]],
--	StackGiven = 3,
	BuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
}

a.spells["Immolate w/ Perfect Aim"] = {
	ID = 348--[[Immolate]],
	Check = function(z)
		return x.NoCC and x.TimeToDie >= 5 and s.BuffDuration(138963--[[Perfect Aim]], "player") > s.CastTime(348--[[Immolate]]) and a.getDoTStat(348--[[Immolate]], "crit_pct") < 100
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Immolate w/ Dark Soul"] = {
	ID = 348--[[Immolate]],
	Check = function(z)
		return x.NoCC and s.BuffDuration(113858--[[Dark Soul: Instability]], "player") > s.CastTime(348--[[Immolate]]) and a.getDoTStat(348--[[Immolate]], "crit_pct") < GetSpellCritChance(3) and x.ImmolateTicksRemain < x.ImmolateTicks / 2
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Immolate w/ Int Proc"] = {
	ID = 348--[[Immolate]],
	Check = function(z)
		local spell_power = a.getDoTStat(348--[[Immolate]], "spell_power")
		local crit_pct = a.getDoTStat(348--[[Immolate]], "crit_pct")
		return x.NoCC and spell_power < GetSpellBonusDamage(3) and crit_pct < GetSpellCritChance(3) and x.ImmolateTicksRemain < x.ImmolateTicks / 2
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Immolate"] = {
	ID = 348--[[Immolate]],
	Check = function(z)
		return x.NoCC and x.ImmolateTicksRemain < s.CastTime(348--[[Immolate]]) and x.TimeToDie >= 5
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Incinerate"] = {
	ID = 29722--[[Incinerate]],
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = true,
	EnemyTargetNeeded = true,
}

a.spells["Drain Soul"] = {
	ID = 1120--[[Drain Soul]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return s.HealthPercent("target") <= 20
	end,
	EnemyTargetNeeded = true,
	FlashColor = "Orange",
}

a.spells["Life Tap w/o buffs under 20%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
	-- buff.dark_soul.down&buff.bloodlust.down&mana.pct<10&target.health.pct<=20
		return x.ActiveEnemy and s.PowerPercent("player", SPELL_POWER_MANA) < 10 and s.HealthPercent("target") <= 20 and not a.Bloodlust() and not s.Buff(113860--[[Dark Soul: Misery]], "player")
	end,
}

a.spells["Life Tap w/o buffs"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
	-- buff.dark_soul.down&buff.bloodlust.down&mana.pct<50
		return x.ActiveEnemy and s.PowerPercent("player", SPELL_POWER_MANA) < 50 and not a.Bloodlust() and not s.Buff(113860--[[Dark Soul: Misery]], "player")
	end,
}

a.spells["Life Tap under 20%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.HealthPercent("target") <= 20
	end,
}

a.spells["Life Tap while moving if mana < 80%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy and s.Moving("player") and s.PowerPercent("player", SPELL_POWER_MANA) < 80 and s.PowerPercent("player", SPELL_POWER_MANA) < s.HealthPercent("target")
	end,
}

a.spells["Life Tap if mana < 60%"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	CheckLast = function(z)
		return s.PowerPercent("player", SPELL_POWER_MANA) < 60
	end,
}

a.spells["Malefic Grasp"] = {
	ID = 103103--[[Malefic Grasp]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Fel Flame while moving"] = {
	ID = 77799--[[Fel Flame]],
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and s.Moving("player") and not s.HasTalent(137587--[[Kil'jaeden's Cunning]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Life Tap"] = {
	ID = 1454--[[Life Tap]],
	NotWhileMoving = false,
	Check = function(z)
		return x.ActiveEnemy
	end,
	NoStopChannel = true,
}

a.spells["Command Demon - Wrathstorm"] = {
	ID = 119915--[[Wrathstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119915--[[Wrathstorm]], 119898--[[Command Demon]]},
}

a.spells["Command Demon - Felstorm"] = {
	ID = 119914--[[Felstorm]],
	Override = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
	FlashID = {119914--[[Felstorm]], 119898--[[Command Demon]]},
}

a.spells["Wrathstorm"] = {
	Type = "pet",
	ID = 115831--[[Wrathstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Wrathstorm")
	end,
	FlashColor = "Yellow",
	Continue = true,
	NoGCD = true,
}

a.spells["Felstorm"] = {
	Type = "pet",
	ID = 89751--[[Felstorm]],
	Check = function(z)
		return x.PetAlive and x.PetActiveEnemy
	end,
	RunLast = function(z)
		a.Flash("Command Demon - Felstorm")
	end,
	FlashColor = "Yellow",
	Continue = true,
	NoGCD = true,
}

a.spells["Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	Buff = 103958--[[Metamorphosis]],
	BuffUnit = "player",
	NotIfActive = true,
	FlashBlink = true,
	Check = function(z)
		return x.ActiveEnemy and x.InCombat
	end,
	CheckLast = function(z)
--		return x.DemonicFury > 150 and ((x.DarkSoulRemains > x.GCD and x.DemonicFury > x.DarkSoulRemains * 32) or x.CorruptionRemaining < 6 or not s.MyDebuff(603--[[Doom]], "target") or x.DemonicFury >= 950 or x.DemonicFury > x.TimeToDie * 32)
		return (x.DarkSoulRemains > x.GCD and x.DemonicFury > x.DarkSoulRemains * 32) or (x.CorruptionDuration < 5 and a.getDoTStat(172--[[Corruption]], "crit_pct") < 100) or x.DoomDuration < x.GCD or x.DemonicFury >= 950 or x.DemonicFury > x.TimeToDie * 32 or x.PerfectAimDuration > x.GCD
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["Cancel Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	FlashBlink = true,
	Override = function(z)
		return x.Metamorphosis and x.DarkSoulRemains <= x.GCD and x.DemonicFury <= 650 and x.TimeToDie > 30
	end,
	FlashColor = "Yellow",	
	NoGCD = true,
}

a.spells["Hand of Gul'dan w/ Perfect Aim"] = {
	ID = 105174--[[Hand of Gul'dan]],
--	MyDebuff = 47960--[[Shadowflame]],
	Check = function(z)
		local travel_time = (s.LastSpellTravelTime(105174--[[Hand of Gul'dan]], "target") or 1.5)
		return x.NoCC and not x.Metamorphosis and x.PerfectAimDuration > x.GCD and x.PerfectAimDuration > travel_time
	end,
	EnemyTargetNeeded = true,
}

a.spells["Hand of Gul'dan"] = {
	ID = 105174--[[Hand of Gul'dan]],
--	MyDebuff = 47960--[[Shadowflame]],
	Check = function(z)
		local travel_time = (s.LastSpellTravelTime(105174--[[Hand of Gul'dan]], "target") or 1.5)
		local shadowflame_remains = s.MyDebuffDuration(47960--[[Shadowflame]], "target")
		local charges, maxCharges, timeLastCast, cooldownDuration = GetSpellCharges(105174--[[Hand of Gul'dan]])
		local recharge_time
		if charges < maxCharges then
			recharge_time = cooldownDuration - GetTime() + timeLastCast
		else
			recharge_time = 0
		end
		local in_flight = s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target")
		return x.NoCC and not x.Metamorphosis and not in_flight and shadowflame_remains < (travel_time + x.ShadowBoltCastTime) and (charges == 2 or shadowflame_remains > travel_time or (charges == 1 and recharge_time < 4))
	end,
	EnemyTargetNeeded = true,
}

a.spells["Soul Fire w/ Perfect Aim"] = {
	ID = 6353--[[Soul Fire]],
	RequireBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		if a.GetConfig("Soul_Fire_Metamorphosis") then
			z.Continue = true
			z.FlashColor = "Pink"
		end
		z.NotWhileMoving = not s.HasTalent(137587--[[Kil'jaeden's Cunning]])
		return x.NoCC and x.Metamorphosis and x.PerfectAimDuration > s.CastTime(6353--[[Soul Fire]])
	end,
	EnemyTargetNeeded = true,
	FlashID = {6353--[[Soul Fire]], 104027--[[Soul Fire]]},
}

a.spells["Soul Fire in Metamorphosis"] = {
	ID = 6353--[[Soul Fire]],
	RequireBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		if a.GetConfig("Soul_Fire_Metamorphosis") then
			z.Continue = true
			z.FlashColor = "Pink"
		end
		z.NotWhileMoving = not s.HasTalent(137587--[[Kil'jaeden's Cunning]])
		return x.NoCC and x.Metamorphosis and x.SoulFireCondition
	end,
	EnemyTargetNeeded = true,
	FlashID = {6353--[[Soul Fire]], 104027--[[Soul Fire]]},
}

a.spells["Soul Fire"] = {
	ID = 6353--[[Soul Fire]],
	RequireBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	Check = function(z)
		z.NotWhileMoving = not s.HasTalent(137587--[[Kil'jaeden's Cunning]])
		if s.HealthPercent("target") < 25 then
			z.FlashColor = "Orange"
		else
			z.FlashColor = "White"
		end
		return x.NoCC and not x.Metamorphosis and x.SoulFireCondition
	end,
	EnemyTargetNeeded = true,
}

a.spells["Doom w/ Perfect Aim"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	FlashNoMacros = false,
	Check = function(z)
		return x.NoCC and x.PerfectAimDuration > x.GCD and (a.getDoTStat(603--[[Doom]], "crit_pct") < 100 or x.DoomTicksRemain <= x.DoomTicks)
	end,
	FlashID = { 124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]] },
	EnemyTargetNeeded = true,
}

a.spells["Doom"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	FlashNoMacros = false,
	Check = function(z)
		local ticks_remain = ceil(s.DebuffDuration(603--[[Doom]], "target") / a.getHastedTickTime(15))
		return x.NoCC and (ticks_remain <= 1 or (ticks_remain + 1 < a.getTotalTicks(4) and x.DarkSoulRemains > x.GCD)) and x.TimeToDie >= 30 and a.getDoTStat(603--[[Doom]], "crit_pct") < 100
	end,
	FlashID = { 124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]] },
	EnemyTargetNeeded = true,
}

a.spells["Touch of Chaos if Corruption < 1.5sec"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and x.CorruptionRemaining > 0 and x.CorruptionRemaining < 1.5
	end,
	EnemyTargetNeeded = true,
	NotWhileMoving = false,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["Touch of Chaos if Corruption < 20sec"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and x.CorruptionRemaining > 0 and x.CorruptionRemaining < 20 and a.getDoTStat(172--[[Corruption]], "crit_pct") < 100
	end,
	EnemyTargetNeeded = true,
	NotWhileMoving = false,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["Touch of Chaos"] = {
	ID = 686--[[Shadow Bolt]],
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	NotWhileMoving = false,
	EnemyTargetNeeded = true,
	FlashID = { 112089--[[Metamorphosis: Touch of Chaos]], 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["Shadow Bolt"] = {
	ID = 686--[[Shadow Bolt]],
	Check = function(z)
		z.NotWhileMoving = not s.HasTalent(137587--[[Kil'jaeden's Cunning]])
		return x.NoCC
	end,
	CheckLast = function(z)
		return not x.Metamorphosis
	end,
	EnemyTargetNeeded = true,
	FlashID = { 686--[[Shadow Bolt]], 112092--[[Shadow Bolt]] },
}

a.spells["Flask of the Warm Sun"] = {
	Type = "item",
	ID = 76085--[[Flask of the Warm Sun]],
	Buff = 105691--[[Flask of the Warm Sun]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.ShouldTemporaryBuff and s.Boss()
	end,
	FlashColor = "Green",
}

a.spells["Mogu Fish Stew"] = {
	Type = "item",
	ID = 74650--[[Mogu Fish Stew]],
	Buff = 104277--[[Well Fed]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	Check = function(z)
		return not x.InCombat and x.ShouldTemporaryBuff and s.Boss()
	end,
	FlashColor = "Green",
}

a.spells["Prepot"] = {
	Type = "item",
	ID = 76093--[[Potion of the Jade Serpent]],
	Buff = 105702--[[Potion of the Jade Serpent]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	NoGCD = true,
	Check = function(z)
		return not x.InCombat and x.ShouldTemporaryBuff and s.Boss()
	end,
	FlashColor = "Pink",
}

a.spells["Potion with Bloodlust or Boss HP <= 20"] = {
	Type = "item",
	ID = 76093--[[Potion of the Jade Serpent]],
	Buff = 105702--[[Potion of the Jade Serpent]],
	BuffUnit = "player",
	EnemyTargetNeeded = true,
	NoGCD = true,
	Check = function(z)
		return x.InCombat and x.ShouldTemporaryBuff and x.ActiveEnemy and s.Boss()
	end,
	CheckLast = function(z)
		return a.Bloodlust() or (s.HealthPercent("target") <= 20)
	end,
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["Carrion Swarm"] = {
	ID = 103967--[[Carrion Swarm]],
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	Interrupt = true,
	FlashColor = "Aqua",
	EnemyTargetNeeded = true,
}

a.spells["Banish"] = {
	ID = 710--[[Banish]],
	EnemyTargetNeeded = 1,
	Check = function(z)
		return x.NoCC
	end,
	CheckLast = function(z)
		return not s.Boss("target")
	end,
	NotWhileMoving = true,
	FlashColor = "Pink",
}

a.spells["Create Healthstone"] = {
	ID = 6201--[[Create Healthstone]],
	Check = function(z)
		return x.ShouldTemporaryBuff and GetItemCount(5512--[[Healthstone]], true) == 0 and not s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Create Soulwell"] = {
	ID = 29893--[[Create Soulwell]],
	Check = function(z)
		return x.ShouldTemporaryBuff  and GetItemCount(5512--[[Healthstone]], true) == 0 and s.InGroup()
	end,
	NotWhileMoving = true,
	FlashColor = "Green",
}

a.spells["Soulshatter"] = {
	ID = 29858--[[Soulshatter]],
	EnemyTargetNeeded = true,
	FlashColor = "Red",
	Check = function(z)
		return x.ThreatPercent >= 80 and s.InGroup()
	end,
}

a.spells["Demonic Leap"] = {
	ID = 109151--[[Demonic Leap]],
	FlashColor = "Red",
	Check = function(z)
		return (s.HasGlyph(56247--[[Glyph of Falling Meteor]]) and IsFalling()) or s.Buff(134370--[[Down Draft]], "target")
	end,
}

a.spells["Curse of Exhaustion"] = {
	ID = 18223--[[Curse of Exhaustion]],
	Check = function(z)
		return not s.Boss("target") and s.Moving("target") and s.EnemyTargetingYou("target")
	end,
	FlashColor = "Red",
	EnemyTargetNeeded = true,
}

a.spells["Dark Regeneration"] = {
	ID = 108359--[[Dark Regeneration]],
	Check = function(z)
		return s.HasTalent(108359--[[Dark Regeneration]]) and s.HealthPercent("player") < 70
	end,
	NoGCD = true,
	FlashColor = "Red",
}

a.spells["Burning Rush"] = {
	ID = 111400--[[Burning Rush]],
	NoRangeCheck = true,
	FlashBlink = true,
	FlashColor = "Aqua",
	Check = function(z)
		local _, runSpeed = GetUnitSpeed("player")
		return s.HasTalent(111400--[[Burning Rush]]) and (runSpeed / 7 * 100 < (100 - 15 * s.DebuffStack(119050--[[Kil'jaeden's Cunning]], "player")) or s.Buff(134370--[[Down Draft]], "target") or s.Buff(123461--[[Get Away!]], "target"))
	end,
}

a.spells["Unbound Will"] = {
	ID = 108482--[[Unbound Will]],
	Check = function(z)
		if not s.HasTalent(108482--[[Unbound Will]]) then
			return false
		end
		local eventIndex = C_LossOfControl.GetNumEvents()
		while (eventIndex > 0) do
			local _, _, _, _, _, timeRemaining = C_LossOfControl.GetEventInfo(eventIndex)
			if timeRemaining > 1.5 then
				return true
			end
			eventIndex = eventIndex - 1
		end
		return s.Debuff(nil, "player", nil, nil, 1)
	end,
	NoGCD = true,
	NoRangeCheck = true,
	FlashBlink = true,
	FlashColor = "Aqua",
}

a.spells["Berserking"] = {
	ID = 26297--[[Berserking]],
	Buff = 26297--[[Berserking]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy)
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["Blood Fury"] = {
	ID = 33702--[[Blood Fury]],
	Buff = 33702--[[Blood Fury]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy)
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["Lifeblood"] = {
	ID = 121279--[[Lifeblood]],
	Buff = 121279--[[Lifeblood]],
	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy)
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["Imp Swarm"] = {
	ID = 104316--[[Imp Swarm]],
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy) and s.HasGlyph(56242--[[Glyph of Imp Swarm]]) and (x.DarkSoulRemains > x.GCD or s.SpellCooldown(113861--[[Dark Soul: Knowledge]]) > (120 / (1 + (UnitSpellHaste("player") / 100))) or x.TimeToDie < 32)
	end,
	NoRangeCheck = true,
}

a.spells["Rain of Fire w/ 2+ enemies"] = {
	ID = 104232--[[Rain of Fire]],
	MyDebuff = 104232--[[Rain of Fire]],
--	Buff = 104232--[[Rain of Fire]],
--	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy) and x.Enemies > 1
	end,
}

a.spells["Rain of Fire"] = {
	ID = 104232--[[Rain of Fire]],
	MyDebuff = 104232--[[Rain of Fire]],
--	BuffUnit = "player",
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy)
	end,
}

a.spells["Havoc"] = {
	ID = 80240--[[Havoc]],
	Buff = 80240--[[Havoc]],
	BuffUnit = "player",
	StackGiven = 3,
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat and (x.ActiveEnemy or x.Dummy) and x.Enemies > 1
	end,
	Continue = true,
	FlashColor = "Purple",
}